!=======================================================================
!
! SUBROUTINE: rkqs.F
!
! PURPOSE   : Fifth-order RK step with monitoring of local truncation error.
!             Adjusts step size if necessary.
!
! SOURCE    : Numerical Recipes, Press et al. 2nd ed. 1992
!
! INPUT     : nvar   : number of ODEs
!             x      : current x (independent variable). See OUTPUT.
!             htry   : step size over which to advance the solution (attempt)
!             eps    : desired accuracy ("Delta_0")
!             y      : current values of ODEs
!             dydx   : derivatives at x (and work array)
!             yscal  : scaling array for errors
!             narg   : number of problem-specific arguments for derivs
!             argarr : problem-specific coefficients for derivs
!             logunit
!
! OUTPUT    : x      : advanced x (by hdid)
!             y      : incremented values of ODEs (new solution)
!             hdid   : actual step size used
!             hnext  : estimated next step size
!
! EXTERNAL  : subroutine derivs(nvar, x, y, narg, argarr, dxdy)
!
! USAGE     : Given y at x, advances solution by step size h using the
!             derivatives in the user-supplied subroutine derivs.
!
!=======================================================================

subroutine rkqs(nvar   ,                  &
                x      , htry   , eps   , &
                y      , dydx   , yscal , &
                narg   , argarr ,         &
                hdid   , hnext  , &
                logunit)

  implicit none

! subroutine arguments
  integer          :: nvar, narg, logunit
  double precision :: x, htry, eps, hdid, hnext
  double precision :: y     (nvar)
  double precision :: dydx  (nvar)
  double precision :: argarr(narg)
  double precision :: yscal (nvar)

! local variables
  logical                       :: done
  integer                       :: i
  double precision              :: errmax, h, htemp , xnew
  double precision, allocatable :: yerr (:)
  double precision, allocatable :: ytemp(:)

  double precision, parameter :: safety = 9.0d-1, errcon = 1.89d-4
  double precision, parameter :: pgrow = -2.0d-1, pshrink = -2.5d-1

!-----------------------------------------------------------------

! allocate local vectors
  allocate(yerr (nvar))
  allocate(ytemp(nvar))

  h    = htry
  done = .false.
  do while (.not. done)
    call rkck(nvar, x, h, y, dydx, narg, argarr, ytemp, yerr, logunit) ! take step
! evaluate accuracy
    errmax = 0d0
    do i = 1, nvar
      errmax = dmax1(errmax,dabs(yerr(i)/yscal(i)))
    enddo
    errmax = errmax / eps ! scale relative to required tolerance
    if (errmax .le. 1d0) then
      done = .true.
    else
! truncation error too large, reduce stepsize
      htemp = safety * h * errmax**pshrink
      h = dsign(max(dabs(htemp),0.1d0*dabs(h)),h)
      xnew = x + h
      if (xnew .eq. x) then
        write(logunit,'("RKQS: step size underflow at x = ",e23.15)') x
        stop
      endif
    endif
  enddo

  if (errmax .gt. errcon) then
    hnext = safety * h * errmax**pgrow
  else
    hnext = 5d0*h
  endif
  hdid = h
  x = x + hdid
  y(:) = ytemp(:)

  deallocate(ytemp)
  deallocate(yerr)

  return
end

!=======================================================================

